// This file is a part of simple_audio
// Copyright (c) 2022-2023 Erikas Taroza <erikastaroza@gmail.com>
//
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation, either version 3 of
// the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License along with this program.
// If not, see <https://www.gnu.org/licenses/>.

/// The playback state of the player.
pub enum PlaybackState
{
    /// The player is currently playing the file.
    Play = 0,
    /// The player is currently paused and there is no output.
    Pause = 1,
    /// The player has finished playing the file.
    Done = 2,
}

/// Provides the current progress of the player.
#[derive(Clone, Copy)]
pub struct ProgressState
{
    /// The position, in seconds, of the player.
    pub position: u64,
    /// The duration, in seconds, of the file that
    /// is being played.
    pub duration: u64,
}

/// Events that are handled in Dart because they
/// cannot be handled in Rust or need user action.
pub enum Callback
{
    /// The media controller trigged the SkipPrev action.
    MediaControlSkipPrev,
    /// The media controller trigged the SkipNext action.
    MediaControlSkipNext,
    /// An error occurred when trying to fetch more bytes for
    /// a network stream.
    NetworkStreamError,
    /// An error occurred when decoding the file.
    DecodeError,
    /// The player is in the looping mode and the playback
    /// just looped to the beginning.
    PlaybackLooped,
    /// The decoder has calculated the duration for the current playback.
    DurationCalculated,
}
